# stores the verbose behavior
config = {
	'verbose': False
}

# defines the attributes for size-statement
size_attrs = [
	'sharex',
	'sharey',
	'width',
	'height',
	'facecolor',
	'edgecolor',
	'linewidth'
]

# defines the attributes for plot-statement
plot_attrs = [
	'label',
	'title',
	'color',
	'xlabel',
	'ylabel',
	'xscale',
	'yscale',
	'xmargin',
	'ymargin',
	'xautoscale',
	'yautoscale',
	'xrotation',
	'yrotation',
	'level',
	'xmajorticks',
	'ymajorticks',
	'xminorticks',
	'yminorticks',
	'xticklabel',
	'yticklabel',
	'gridlines',
	'facecolor',
	'marker',
	'linestyle',
	'linewidth',
	'alpha',
	'dashcapstyle',
	'dashjoinstyle',
	'solidcapstyle',
	'solidjoinstyle',
	'drawstyle',
	'fillstyle',
	'markeredgecolor',
	'markeredgewidth',
	'markerfacecolor',
	'markersize'
]
